@@ -23,6 +23,8 @@ from . import template as _template
 from .util import resolve_ndsserver
 from ._qt import create_app
 from .scope import NDScope
+from .exceptions import UnknownChannelError
+
 
 logging.addLevelName(5, 'DATA')
 logging.basicConfig(
@@ -168,46 +170,77 @@ parser = argparse.ArgumentParser(
     add_help=False,
 )
 
-parser.add_argument('channels', nargs='*',
-                    help=argparse.SUPPRESS)
-parser.add_argument('-t', '--time', action=TimeParseAction, default=[],
-                    help="time boundary (GPS or natural language), may specify one or two)")
-parser.add_argument('-w', '--window', '--time-window', dest='time_window',
-                    help="time window scalar or tuple, in seconds")
+parser.add_argument(
+    'channels', nargs='*',
+    help=argparse.SUPPRESS)
+parser.add_argument(
+    '-t', '--time', action=TimeParseAction, default=[],
+    help="time boundary (GPS or natural language), may specify one or two)")
+parser.add_argument(
+    '-w', '--window', '--time-window', dest='time_window',
+    help="time window scalar or tuple, in seconds")
+tgroup = parser.add_mutually_exclusive_group()
+tgroup.add_argument(
+    '--trend', choices=['auto', 'raw', 'sec', 'min'],
+    help="specify trend type, 'auto' auto-selects best trend for time span [default]")
 lgroup = parser.add_mutually_exclusive_group()
-lgroup.add_argument('-g', '--grid', dest='layout', action='store_const', const='grid',
-                    help="arrange channels in a grid of plots (default)")
-lgroup.add_argument('-k', '--stack', dest='layout', action='store_const', const='stack',
-                    help="arrange channels in a vertical stack of plots")
-lgroup.add_argument('-s', '--single', dest='layout', action='store_const', const='single',
-                    help="all channels in a single plot")
-lgroup.add_argument('-l', '--table', dest='layout', action='store_const', const='table',
-                    help="subplot table layout (period/comman in channel list starts new colum/row)")
-# parser.add_argument('--colspan', action='store_true',
-#                     help="expand subplots to fill empty columns (BUGGY!)")
-parser.add_argument('--title', '--window-title', dest='window_title', metavar='TITLE',
-                    help="application window title")
-parser.add_argument('--light', dest='color_mode', action='store_const', const='light',
-                    help="light color mode (black-on-white plots)")
-parser.add_argument('--dark', dest='color_mode', action='store_const', const='dark',
-                    help="dark color mode (white-on-black plots) [default]")
-parser.add_argument('-bw', '--black-on-white', action='store_true',
-                    help=argparse.SUPPRESS)
-parser.add_argument('--size', nargs=2, type=int, metavar=('WIDTH', 'HEIGHT'),
-                    help="specify initial window size")
-parser.add_argument('--nds', metavar='HOST[:PORT]',
-                    help=f"NDS server [{const.NDSSERVER}]")
-parser.add_argument('--gen-template', action='store_true',
-                    help="generate YAML template, dump to stdout, and exit")
-parser.add_argument('--single-shot', metavar='PATH',
-                    help="export initial plot/data then exit (see export options in usage)")
-parser.add_argument('--version', action='version', version=full_version(),
-                    help="print version and exit")
-parser.add_argument('-h', '--help', action='help',
-                    help=argparse.SUPPRESS)
-parser.add_argument('--usage', action='store_true',
-                    help="print more detailed usage information and exit")
-                    #help=argparse.SUPPRESS)
+lgroup.add_argument(
+    '-g', '--grid', dest='layout', action='store_const', const='grid',
+    help="arrange channels in a grid of plots (default)")
+lgroup.add_argument(
+    '-k', '--stack', dest='layout', action='store_const', const='stack',
+    help="arrange channels in a vertical stack of plots")
+lgroup.add_argument(
+    '-s', '--single', dest='layout', action='store_const', const='single',
+    help="all channels in a single plot")
+lgroup.add_argument(
+    '-l', '--table', dest='layout', action='store_const', const='table',
+    help="subplot table layout (period/comman in channel list starts new colum/row)")
+# parser.add_argument(
+#     '--colspan', action='store_true',
+#     help="expand subplots to fill empty columns (BUGGY!)")
+parser.add_argument(
+    '--title', '--window-title', dest='window_title', metavar='TITLE',
+    help="application window title")
+parser.add_argument(
+    '--light', dest='color_mode', action='store_const', const='light',
+    help="light color mode (black-on-white plots)")
+parser.add_argument(
+    '--dark', dest='color_mode', action='store_const', const='dark',
+    help="dark color mode (white-on-black plots) [default]")
+parser.add_argument(
+    '-bw', '--black-on-white', action='store_true',
+    help=argparse.SUPPRESS)
+parser.add_argument(
+    '--font-size', type=int, metavar='SIZE',
+    help="label font size")
+parser.add_argument(
+    '--line-width', type=int, metavar='WIDTH',
+    help="curve line width")
+parser.add_argument(
+    '--size', nargs=2, type=int, metavar=('WIDTH', 'HEIGHT'),
+    help="specify initial window size")
+parser.add_argument(
+    '--expand', action='store_true',
+    help="expand control bar")
+parser.add_argument(
+    '--nds', metavar='HOST[:PORT]',
+    help=f"NDS server [{const.NDSSERVER}]")
+parser.add_argument(
+    '--gen-template', action='store_true',
+    help="generate YAML template, dump to stdout, and exit")
+parser.add_argument(
+    '--single-shot', metavar='PATH',
+    help="export initial plot/data then exit (see export options in usage)")
+parser.add_argument(
+    '--version', action='version', version=full_version(),
+    help="print version and exit")
+parser.add_argument(
+    '-h', '--help', action='help',
+    help=argparse.SUPPRESS)
+parser.add_argument(
+    '--usage', action='store_true',
+    help="print more detailed usage information and exit")
 
 
 def main():
@@ -259,12 +292,27 @@ def main():
     if args.layout and args.layout != ltype:
         _template.convert_layout(template, args.layout)
 
+    if args.trend:
+        template['trend'] = args.trend
+
     if args.color_mode:
         template['color-mode'] = args.color_mode
 
     if args.window_title:
         template['window-title'] = args.window_title
 
+    if args.font_size:
+        if args.font_size > 0:
+            template['font-size'] = args.font_size
+        else:
+            parser.error('Font size must be greater than 0.')
+
+    if args.line_width:
+        if args.line_width > 0:
+            template['line-width'] = args.line_width
+        else:
+            parser.error('Line width must be greater than 0.')
+
     ##########
     # parse time specs
 
@@ -323,7 +371,10 @@ def main():
     app = create_app()
 
     scope = NDScope()
-    scope.load_template(template)
+    try:
+        scope.load_template(template)
+    except UnknownChannelError as e:
+        sys.exit(10)
     if not scope.data.channels:
         if args.single_shot:
             parser.error("Must specify channels for single shot mode.")
@@ -335,6 +386,8 @@ def main():
     scope.show()
     if args.size:
         scope.resize(*args.size)
+    if args.expand:
+        scope.control_expand()
     sys.exit(app.exec_())
 
 
