@@ -44,166 +44,128 @@ def filter_channel(channel, show_slow, show_fast, show_online_only):
 
 # ------------------------------------------------------------------------------
 
-# modified from https://github.com/pyside/Examples/blob/master/examples/itemviews/simpletreemodel/simpletreemodel.py
-
-
-class AvailableChannelTreeBranchItem:
-    def __init__(self, column_data, parent=None):
-        self.column_data = column_data
+class AvailableChannelTreeItem:
+    def __init__(self, parent=None, data=None, is_leaf=False):
         self.parent = parent
-        self.flags = (QtCore.Qt.ItemIsEnabled, QtCore.Qt.NoItemFlags, QtCore.Qt.NoItemFlags)
-        self.channel_name_part_dict = {}
-        self.channel_name_dict = {}
-        self.has_slow = False
-        self.has_slow_online = False
-        self.has_fast = False
-        self.has_fast_online = False
-        self.is_leaf = False
+        self.data = data
+        self.is_leaf = is_leaf
+        if not is_leaf:
+            self.branch_dict = {}
+            self.leaf_dict = {}
 
-    def getChild(self, row):
-        return (list(self.channel_name_part_dict.values()) + list(self.channel_name_dict.values()))[row]
+            self.has_slow = False
+            self.has_slow_online = False
+            self.has_fast = False
+            self.has_fast_online = False
 
-    def getChildCount(self):
-        return len(self.channel_name_part_dict) + len(self.channel_name_dict)
+    def child_list(self):
+        if not self.is_leaf:
+            return list(self.branch_dict.values()) + list(self.leaf_dict.values())
+        return []
 
-    def getColumnCount(self):
-        return len(self.column_data)
-
-    def getData(self, column, role):
-        if role == QtCore.Qt.DisplayRole:
-            try:
-                return self.column_data[column]
-            except IndexError:
-                return QtCore.QVariant()
-        else:
-            return QtCore.QVariant()
-
-    def getFlags(self, column):
-        try:
-            return self.flags[column]
-        except IndexError:
-            return QtCore.Qt.NoItemFlags
-
-    def getParent(self):
-        return self.parent
-
-    def getRow(self):
+    def row(self):
         if self.parent:
-            return (list(self.parent.channel_name_part_dict.values()) + list(self.parent.channel_name_dict.values())).index(self)
-        else:
-            return 0
-
-
-class AvailableChannelTreeLeafItem:
-    def __init__(self, channel, parent):
-        self.channel = channel
-        self.parent = parent
-        self.column_data = (channel.name, str(channel.sample_rate))
-        self.flags = (
-            (QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDragEnabled | QtCore.Qt.ItemNeverHasChildren),
-            (QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemNeverHasChildren),
-        )
-        self.is_leaf = True
-
-    def getChild(self, row):
-        return None
-
-    def getChildCount(self):
-        return 0
-
-    def getColumnCount(self):
-        return len(self.column_data)
-
-    def getData(self, column, role):
-        if role == QtCore.Qt.DisplayRole:
-            try:
-                return self.column_data[column]
-            except IndexError:
-                return QtCore.QVariant()
-        elif role == QtCore.Qt.ForegroundRole:
-            return brush_for_channel(self.channel)
-        else:
-            return QtCore.QVariant()
-
-    def getFlags(self, column):
-        try:
-            return self.flags[column]
-        except IndexError:
-            return QtCore.Qt.NoItemFlags
-
-    def getParent(self):
-        return self.parent
-
-    def getRow(self):
-        if self.parent:
-            return (list(self.parent.channel_name_part_dict.values()) + list(self.parent.channel_name_dict.values())).index(self)
+            return self.parent.child_list().index(self)
         else:
             return 0
 
 
 class AvailableChannelTreeModel(QtCore.QAbstractItemModel):
-    def __init__(self, channel_list, max_depth=4, parent=None):
-        super().__init__(parent)
-        column_data = ('name', 'rate')
-        self.root = AvailableChannelTreeBranchItem(column_data)
+    def __init__(self, channel_list, max_depth=4):
+        super().__init__()
+        self.header_role_data = {QtCore.Qt.DisplayRole: ('name', 'rate')}
+        self.root = AvailableChannelTreeItem()
         self.max_depth = max_depth
+        self.insert(channel_list)
+
+    def insert(self, channel_list):
+        split_re = re.compile('[:\-_]')
         for channel in channel_list:
-            self.insert(channel)
+            current = self.root
+            slow = is_slow(channel.sample_rate)
+            online = channel.online
+            name_part_list = split_re.split(channel.name, self.max_depth)
+            for name_part in name_part_list[:-1]:
+                if name_part not in current.branch_dict:
+                    current.branch_dict[name_part] = AvailableChannelTreeItem(current, name_part)
+                current = current.branch_dict[name_part]
+                if slow:
+                    current.has_slow = True
+                    if online:
+                        current.has_slow_online = True
+                else:
+                    current.has_fast = True
+                    if online:
+                        current.has_fast_online = True
+            current.leaf_dict[channel.name] = AvailableChannelTreeItem(current, channel, is_leaf=True)
 
-    def insert(self, channel):
-        current = self.root
-        channel_name_part_list = re.split('[:\-_]', channel.name, self.max_depth)
-        for channel_name_part in channel_name_part_list[:-1]:
-            column_data = (channel_name_part, '')
-            current = current.channel_name_part_dict.setdefault(channel_name_part, AvailableChannelTreeBranchItem(column_data, current))
-            if is_slow(channel.sample_rate):
-                current.has_slow = True
-                if channel.online:
-                    current.has_slow_online = True
-            else:
-                current.has_fast = True
-                if channel.online:
-                    current.has_fast_online = True
-        current.channel_name_dict[channel.name] = AvailableChannelTreeLeafItem(channel, current)
+    def columnCount(self, parent=QtCore.QModelIndex()):
+        return len(self.header_role_data[QtCore.Qt.DisplayRole])
 
-    def columnCount(self, parent):
-        if parent.isValid():
-            return parent.internalPointer().getColumnCount()
-        return self.root.getColumnCount()
-
-    def data(self, index, role):
+    def data(self, index, role=QtCore.Qt.DisplayRole):
         if not index.isValid():
             return QtCore.QVariant()
-        if role in [QtCore.Qt.DisplayRole, QtCore.Qt.ForegroundRole]:
-            item = index.internalPointer()
-            return item.getData(index.column(), role)
-        return QtCore.QVariant()
+
+        item = index.internalPointer()
+        if item.is_leaf:
+            channel = item.data
+            brush = brush_for_channel(channel)
+            sample_rate = f'{float(channel.sample_rate):g}'
+            role_data = {QtCore.Qt.DisplayRole: (channel.name, sample_rate),
+                         QtCore.Qt.ForegroundRole: (brush, brush)}
+        else:
+            name_part = item.data
+            role_data = {QtCore.Qt.DisplayRole: (name_part, '')}
+
+        try:
+            return role_data[role][index.column()]
+        except KeyError:
+            return QtCore.QVariant()
+        except IndexError:
+            return QtCore.QVariant()
 
     def flags(self, index):
         if not index.isValid():
             return QtCore.Qt.NoItemFlags
-        item = index.internalPointer()
-        return item.getFlags(index.column())
 
-    def headerData(self, section, orientation, role):
-        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
-            return self.root.column_data[section]
+        item = index.internalPointer()
+        if item.is_leaf:
+            flags = (
+                QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDragEnabled | QtCore.Qt.ItemNeverHasChildren,
+                QtCore.Qt.ItemIsEnabled)
+        else:
+            flags = (QtCore.Qt.ItemIsEnabled, QtCore.Qt.ItemIsEnabled)
+
+        try:
+            return flags[index.column()]
+        except IndexError:
+            return QtCore.Qt.NoItemFlags
+
+    def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
+        if orientation == QtCore.Qt.Horizontal:
+            try:
+                return self.header_role_data[role][section]
+            except KeyError:
+                return QtCore.QVariant()
+            except IndexError:
+                return QtCore.QVariant()
+
         return QtCore.QVariant()
 
-    def index(self, row, column, parent):
+    def index(self, row, column, parent=QtCore.QModelIndex()):
         if not self.hasIndex(row, column, parent):
             return QtCore.QModelIndex()
+
         if not parent.isValid():
             parent_item = self.root
         else:
             parent_item = parent.internalPointer()
-        child_item = parent_item.getChild(row)
-        if child_item:
-            return self.createIndex(row, column, child_item)
-        return QtCore.QModelIndex()
 
-    def itemFromIndex(self, index):
-        return index.internalPointer()
+        try:
+            child_item = parent_item.child_list()[row]
+            return self.createIndex(row, column, child_item)
+        except IndexError:
+            return QtCore.QModelIndex()
 
     def mimeData(self, indexes):
         text_list = [self.data(index, QtCore.Qt.DisplayRole) for index in indexes if index.isValid()]
@@ -218,20 +180,25 @@ class AvailableChannelTreeModel(QtCore.QAbstractItemModel):
     def parent(self, index):
         if not index.isValid():
             return QtCore.QModelIndex()
+
         child_item = index.internalPointer()
-        parent_item = child_item.getParent()
+        parent_item = child_item.parent
         if parent_item == self.root:
             return QtCore.QModelIndex()
-        return self.createIndex(parent_item.getRow(), 0, parent_item)
 
-    def rowCount(self, parent):
+        row = parent_item.row()
+        return self.createIndex(row, 0, parent_item)
+
+    def rowCount(self, parent=QtCore.QModelIndex()):
         if parent.column() > 0:
             return 0
+
         if not parent.isValid():
             parent_item = self.root
         else:
             parent_item = parent.internalPointer()
-        return parent_item.getChildCount()
+
+        return len(parent_item.child_list())
 
 
 # Filters the branch and leaf items based on the sample rate and online status
@@ -247,7 +214,7 @@ class AvailableChannelTreeSortFilterProxyModel(QtCore.QSortFilterProxyModel):
         index = self.sourceModel().index(source_row, 0, source_parent)
         item = index.internalPointer()
         if item.is_leaf:
-            channel = item.channel
+            channel = item.data
             return filter_channel(channel, self.show_slow, self.show_fast, self.show_online_only)
         else:
             if self.show_online_only:
@@ -775,9 +742,10 @@ class ChannelSelectDialog(*load_ui('channel_select_dialog.ui')):
     def available_channel_tree_view_activated_slot(self, index):
         if not self.channel_list_widget.available_channel_tree_proxy_model.hasChildren(index) and index.column() == 0:
             source_index = self.channel_list_widget.available_channel_tree_proxy_model.mapToSource(index)
-            item = self.channel_list_widget.available_channel_tree_proxy_model.sourceModel().itemFromIndex(source_index)
+            item = source_index.internalPointer()
             if item.is_leaf:
-                self.channel_select_widget.add_channel(item.channel.name)
+                channel = item.data
+                self.channel_select_widget.add_channel(channel.name)
 
     # Appends the item in the available channel table at the supplied
     # index to the end of the selected channel table.
