@@ -148,6 +148,7 @@ TICK_DATE_FMT = '%b %d %Y\n%H:%M:%S %Z'
 LABEL_FILL = (0, 0, 0, 200)
 
 
+# foreground/background color for light/dark mode styles
 COLOR_MODE = {
     'light': {
         'fg': QColor('black'),
@@ -158,3 +159,7 @@ COLOR_MODE = {
         'bg': QBrush(QColor('black')),
     },
 }
+
+
+# alpha value for the trend min/max traces
+TREND_MINMAX_ALPHA = 120
