@@ -32,6 +32,8 @@ class Crosshair(QtCore.QObject):
             fill=LABEL_FILL,
         )
         self.active_plot = None
+        self.pos = None
+        self.t0 = None
 
     def set_font(self, font):
         """Set text label font"""
@@ -54,13 +56,15 @@ class Crosshair(QtCore.QObject):
             self.active_plot.removeItem(self.label)
             self.active_plot = None
         if plot:
-            plot.addItem(self.hline)
-            plot.addItem(self.vline)
-            plot.addItem(self.label)
+            plot.addItem(self.hline, ignoreBounds=True)
+            plot.addItem(self.vline, ignoreBounds=True)
+            plot.addItem(self.label, ignoreBounds=True)
             self.active_plot = plot
 
-    def update(self, pos, plot, t0):
+    def update(self, plot, pos, t0):
         self.set_active_plot(plot)
+        self.pos = pos
+        self.t0 = t0
         ppos = plot.vb.mapSceneToView(pos)
         x = ppos.x()
         y = ppos.y()
@@ -73,11 +77,12 @@ class Crosshair(QtCore.QObject):
             ay = 1
         else:
             ay = 0
-        t = t0 + x
         self.hline.setPos(y)
         self.vline.setPos(x)
         self.label.setPos(x, y)
         self.label.setAnchor((ax, ay))
+        t = t0 + x
+        y = plot.y_pos_to_val(y)
         fmt = '%Y/%m/%d %H:%M:%S %Z'
         gt = util.gpstime_parse(t)
         greg_utc = gt.astimezone(tzutc()).strftime(fmt)
@@ -97,3 +102,7 @@ class Crosshair(QtCore.QObject):
         x += dt
         self.vline.setPos(x)
         self.label.setPos(x, y)
+
+    def redraw(self):
+        if self.active_plot and self.pos and self.t0:
+            self.update(self.active_plot, self.pos, self.t0)
