@@ -1,4 +1,5 @@
 import copy
+import logging
 import collections
 
 import numpy as np
@@ -17,6 +18,7 @@ from . import template
 from . import legend
 from . import cursors
 
+logger = logging.getLogger('PLOT')
 
 ##################################################
 
@@ -115,12 +117,13 @@ class TimeStringAxis(pg.AxisItem):
 
 class NDScopePlot(PlotItem):
     channel_select_request = Signal('PyQt_PyObject')
-    channel_add_complete = Signal()
+    request_reset = Signal()
     new_plot_request = Signal('PyQt_PyObject')
     remove_plot_request = Signal('PyQt_PyObject')
     t0_reset = Signal(float)
     t_cursors_enabled = Signal(bool)
     t_cursor_moved = Signal('PyQt_PyObject')
+    log_mode_toggled = Signal('PyQt_PyObject')
 
     def __init__(self, data_store, *args, loc=None):
         """Initialize NDSCope Plot object
@@ -171,8 +174,9 @@ class NDScopePlot(PlotItem):
         # setting the font.
         self.titleLabel.opts.pop('size')
 
-        self.t_cursors = None
-        self.y_cursors = None
+        self.t_cursors = cursors.TCursors(self)
+        self.t_cursors.cursor_moved.connect(self._update_t_cursor)
+        self.y_cursors = cursors.YCursors(self)
 
     def dropEvent(self, event):
         data = event.mimeData()
@@ -194,10 +198,7 @@ class NDScopePlot(PlotItem):
 
     def _add_channel_obj(self, cc):
         channel = cc.channel
-        try:
-            self.data.add_channel(channel)
-        except ValueError:
-            return
+        self.data.add_channel(channel)
         self.channels[channel] = cc
         self.addItem(cc.curves['y'])
         self.addItem(cc.curves['bad'])
@@ -212,7 +213,7 @@ class NDScopePlot(PlotItem):
 
     def _add_channel_menu(self, channel):
         self.add_channel(channel)
-        self.channel_add_complete.emit()
+        self.request_reset.emit()
 
     def remove_channel(self, channel):
         """remove channel from plot
@@ -241,14 +242,10 @@ class NDScopePlot(PlotItem):
         """
         set_chans = set(chan_obj_list)
         cur_chans = set(self.channels.values())
-        added = False
         for chan in cur_chans - set_chans:
             self._remove_channel_obj(chan)
         for chan in set_chans - cur_chans:
             self._add_channel_obj(chan)
-            added = True
-        if added:
-            self.channel_add_complete.emit()
 
     def set_font(self, font):
         """Set label and axis font"""
@@ -257,10 +254,8 @@ class NDScopePlot(PlotItem):
         self.axes['left']['item'].label.setFont(font)
         self.axes['bottom']['item'].setTickFont(font)
         self.axes['left']['item'].setTickFont(font)
-        if self.t_cursors:
-            self.t_cursors.set_font(font)
-        if self.y_cursors:
-            self.y_cursors.set_font(font)
+        self.t_cursors.set_font(font)
+        self.y_cursors.set_font(font)
 
     def set_color_mode(self, mode):
         fg = const.COLOR_MODE[mode]['fg']
@@ -270,34 +265,95 @@ class NDScopePlot(PlotItem):
         self.legend.setTextColor(fg)
         self.axes['left']['item'].set_color_mode(mode)
         self.axes['bottom']['item'].set_color_mode(mode)
-        if self.t_cursors:
-            self.t_cursors.set_color_mode(mode)
-        if self.y_cursors:
-            self.y_cursors.set_color_mode(mode)
-        self._color_mode = mode
+        self.t_cursors.set_color_mode(mode)
+        self.y_cursors.set_color_mode(mode)
+
+    ##### y axis
+
+    @property
+    def log_mode(self):
+        """current log mode state"""
+        return self.getAxis('left').logMode
+
+    def y_pos_to_val(self, y):
+        """get the y coordinate position for a given value
+
+        Takes into account logarithmic axis scaling.
+
+        """
+        if self.log_mode:
+            y = 10 ** y
+        return y
+
+    def y_val_to_pos(self, y):
+        """get the y value for a given coordinate position
+
+        Takes into account logarithmic axis scaling.  If the value
+        doesn't correspond to any position on the axis, None will be
+        returned.
+
+        """
+        if self.log_mode:
+            if y > 0:
+                # HACK: we convert to normal float because pyyaml
+                # can't handle numpy objects
+                return float(np.log10(y))
+            else:
+                return None
+        else:
+            return y
+
+    def set_y_range(self, y_range):
+        """set the Y axis range
+
+        If None or "auto" then auto range will be enabled, otherwise
+        range should be tuple.
+
+        """
+        logger.debug(f"set Y range: {y_range}")
+        if y_range in [None, 'auto']:
+            self.enableAutoRange(axis='y')
+        else:
+            self.disableAutoRange(axis='y')
+            y_range = tuple(map(self.y_val_to_pos, y_range))
+            self.getViewBox().setYRange(*y_range, padding=0.0)
+
+    def set_log_mode(self, log=True):
+        """set the Y scale to be log
+
+        True turns on log mode, False turns it off.
+
+        """
+        assert isinstance(log, bool)
+        # FIXME HACK: This is a hack around
+        # https://github.com/pyqtgraph/pyqtgraph/issues/2307 Since we
+        # can not stop it from autoscaling the range at least once
+        # when we switch on log mode, we keep track of the axis state
+        # and set it back after the change.
+        vb = self.getViewBox()
+        x_range, y_range = vb.viewRange()
+        self.setLogMode(y=log)
+        vb.disableAutoRange(vb.XAxis)
+        vb.setXRange(min=x_range[0], max=x_range[1], padding=0.0)
+        self.y_cursors.redraw()
+        self.log_mode_toggled.emit(self)
+
+    ##########
 
     def _reset_t0(self, val):
         self.t0_reset.emit(val)
 
-    def enable_t_cursors(self):
-        """enable T axis cursors"""
-        if self.t_cursors:
-            return
-        self.t_cursors = cursors.TCursors()
-        self.t_cursors.cursor_moved.connect(self._update_t_cursor)
-        self.addItem(self.t_cursors.T1)
-        self.addItem(self.t_cursors.T2)
-        self.addItem(self.t_cursors.diff)
-        self.t_cursors.set_color_mode(self._color_mode)
+    ##### cursors
 
-    def disable_t_cursors(self):
-        """disable T axis cursors"""
-        if not self.t_cursors:
-            return
-        self.removeItem(self.t_cursors.T1)
-        self.removeItem(self.t_cursors.T2)
-        self.removeItem(self.t_cursors.diff)
-        self.t_cursors = None
+    def enable_t_cursors(self):
+        """enable T cursors and return cursor object"""
+        if self.t_cursors.T1 not in self.getViewBox().allChildren():
+            logger.debug("T cursor enable")
+            self.addItem(self.t_cursors.T1, ignoreBounds=True)
+            self.addItem(self.t_cursors.T2, ignoreBounds=True)
+            self.addItem(self.t_cursors.diff, ignoreBounds=True)
+            self.t_cursors.reset()
+        return self.t_cursors
 
     def _update_t_cursors(self, enable):
         self.t_cursors_enabled.emit(enable)
@@ -306,24 +362,16 @@ class NDScopePlot(PlotItem):
         self.t_cursor_moved.emit(indval)
 
     def enable_y_cursors(self):
-        """enable Y axis cursors"""
-        if self.y_cursors:
-            return
-        self.y_cursors = cursors.YCursors()
-        self.addItem(self.y_cursors.Y1)
-        self.addItem(self.y_cursors.Y2)
-        self.addItem(self.y_cursors.diff)
-        self.y_cursors.reset(self)
-        self.y_cursors.set_color_mode(self._color_mode)
+        """enable Y cursors and return cursor object"""
+        if self.y_cursors.Y1 not in self.getViewBox().allChildren():
+            logger.debug("Y cursor enable")
+            self.addItem(self.y_cursors.Y1, ignoreBounds=True)
+            self.addItem(self.y_cursors.Y2, ignoreBounds=True)
+            self.addItem(self.y_cursors.diff, ignoreBounds=True)
+            self.y_cursors.reset()
+        return self.y_cursors
 
-    def disable_y_cursors(self):
-        """disable Y axis cursors"""
-        if not self.y_cursors:
-            return
-        self.removeItem(self.y_cursors.Y1)
-        self.removeItem(self.y_cursors.Y2)
-        self.removeItem(self.y_cursors.diff)
-        self.y_cursors = None
+    ##########
 
     # SLOT
     def _update_legend_item(self, channel):
@@ -357,6 +405,18 @@ class NDScopePlot(PlotItem):
             #xMax=gpsnow()-t0+1,
         )
 
+    def set_all_mean_curves_visibility(self, visible):
+        for cc in self.channels.values():
+            cc.set_mean_curve_visibility(visible)
+
+    def set_all_min_curves_visibility(self, visible):
+        for cc in self.channels.values():
+            cc.set_min_curve_visibility(visible)
+
+    def set_all_max_curves_visibility(self, visible):
+        for cc in self.channels.values():
+            cc.set_max_curve_visibility(visible)
+
     def update(self, data, t0):
         """update all channel
 
@@ -508,7 +568,10 @@ class NDScopePlotChannel(QtCore.QObject):
         for param, value in params.items():
             if param not in PARAMS:
                 raise KeyError(f"invalid parameter '{param}', options are: {PARAMS}")
-            self.params[param] = value
+            if param == 'color':
+                self.params[param] = pg.mkColor(value).name()
+            else:
+                self.params[param] = value
 
         color = self.get_QColor()
         pen = pg.mkPen(color, width=self.params['width'])
@@ -524,6 +587,15 @@ class NDScopePlotChannel(QtCore.QObject):
         self._update_label()
         self.redraw_data()
 
+    def set_mean_curve_visibility(self, visible):
+        self.curves['y'].setVisible(visible)
+
+    def set_min_curve_visibility(self, visible):
+        self.curves['min'].setVisible(visible)
+
+    def set_max_curve_visibility(self, visible):
+        self.curves['max'].setVisible(visible)
+
     def get_QColor(self):
         """Get channel color as a QColor object"""
         return pg.mkColor(self.params['color'])
@@ -575,11 +647,13 @@ class NDScopePlotChannel(QtCore.QObject):
             # that case there's nothing we need to convert, so
             # just pass
             pass
+        y = self.transform(y)
         self.curves[mod].setData(
             x=t,
-            y=self.transform(y),
+            y=y,
             connect="finite",
         )
+        return y
 
     def set_data(self, data, t0):
         """set data for curves
@@ -590,14 +664,12 @@ class NDScopePlotChannel(QtCore.QObject):
         t = data.tarray - t0
 
         if 'raw' in data:
-            y = data['raw']
+            y = self._set_curve_data('y', t, data['raw'])
         else:
-            y = data['mean']
+            y = self._set_curve_data('y', t, data['mean'])
             for mod in ['min', 'max']:
                 self._set_curve_data(mod, t, data[mod])
 
-        self._set_curve_data('y', t, y)
-
         self.curves['bad'].setData(
             x=t,
             y=hold_over_nan(y),
