@@ -1,5 +1,5 @@
 import weakref
-import logging
+from contextlib import contextmanager
 
 from qtpy import QtCore, QtGui, QtWidgets
 
@@ -9,20 +9,67 @@ from .const import CHANNEL_REGEXP, CHANNEL_RE
 
 AxisCtrlTemplate, __ = load_ui('axisCtrlTemplate.ui')
 
-class AxisCtrlMenu(QtWidgets.QMenu, AxisCtrlTemplate):
+class AxisCtrlMenuItem(QtWidgets.QMenu, AxisCtrlTemplate):
     def __init__(self, title, mainmenu):
         super().__init__(title, mainmenu)
         self.setupUi(self)
-        self.manualRadio.clicked.connect(mainmenu.yManualClicked)
         self.minText.setValidator(QtGui.QDoubleValidator())
-        self.minText.editingFinished.connect(mainmenu.yRangeTextChanged)
         self.maxText.setValidator(QtGui.QDoubleValidator())
-        self.maxText.editingFinished.connect(mainmenu.yRangeTextChanged)
-        self.autoRadio.clicked.connect(mainmenu.yAutoClicked)
-        self.autoPercentSpin.valueChanged.connect(mainmenu.yAutoSpinChanged)
-        # self.mouseCheck.toggled.connect(mainmenu.yMouseToggled)
-        # self.visibleOnlyCheck.toggled.connect(mainmenu.yVisibleOnlyToggled)
-        # self.autoPanCheck.toggled.connect(mainmenu.yAutoPanToggled)
+
+    @property
+    def _controls(self):
+        return [
+            self.manualRadio,
+            self.minText,
+            self.maxText,
+            self.autoRadio,
+            self.autoPercentSpin,
+            self.logModeCheck,
+        ]
+
+    def blockSignals(self, block):
+        for c in self._controls:
+            c.blockSignals(block)
+
+    @contextmanager
+    def signal_blocker(self):
+        self.blockSignals(True)
+        try:
+            yield
+        finally:
+            self.blockSignals(False)
+
+
+class MouseModeMenuItem(QtWidgets.QMenu):
+    def __init__(self, title, mainmenu):
+        super().__init__(title, mainmenu)
+        group = QtWidgets.QActionGroup(self)
+        self.pan = QtWidgets.QAction("pan/zoom", self)
+        self.rect = QtWidgets.QAction("zoom box", self)
+        self.addAction(self.pan)
+        self.addAction(self.rect)
+        self.pan.setCheckable(True)
+        self.rect.setCheckable(True)
+        self.pan.setActionGroup(group)
+        self.rect.setActionGroup(group)
+
+
+class CursorWidget(QtWidgets.QWidget):
+    def __init__(self, check1, check2):
+        super().__init__()
+        self._c1 = QtWidgets.QCheckBox(check1)
+        self._c1.setToolTip(f"enable {check1} cursor")
+        self._c2 = QtWidgets.QCheckBox(check2)
+        self._c2.setToolTip(f"enable {check2} cursor")
+        setattr(self, check1, self._c1)
+        setattr(self, check2, self._c2)
+        self.reset = QtWidgets.QPushButton("reset")
+        self.reset.setToolTip("reset cursor positions")
+        self.layout = QtWidgets.QHBoxLayout()
+        self.layout.addWidget(self._c1)
+        self.layout.addWidget(self._c2)
+        self.layout.addWidget(self.reset)
+        self.setLayout(self.layout)
 
 
 # this is lifted from the pqtgraph.ViewBoxMenu module
@@ -41,7 +88,6 @@ class NDScopePlotMenu(QtWidgets.QMenu):
         self.addLabel(title)
         self.addSeparator()
 
-        # view all data
         self.viewAll = QtWidgets.QAction("view all data", self)
         self.viewAll.triggered.connect(self.autoRange)
         self.addAction(self.viewAll)
@@ -50,42 +96,47 @@ class NDScopePlotMenu(QtWidgets.QMenu):
         self.resetT0.triggered.connect(self.reset_t0)
         self.addAction(self.resetT0)
 
-        self.yAxisUI = AxisCtrlMenu("Y axis scale", self)
+        self.yAxisUI = AxisCtrlMenuItem("Y axis scale", self)
+        self.yAxisUI.manualRadio.clicked.connect(self.yManualClicked)
+        self.yAxisUI.minText.editingFinished.connect(self.yRangeTextChanged)
+        self.yAxisUI.maxText.editingFinished.connect(self.yRangeTextChanged)
+        self.yAxisUI.autoRadio.clicked.connect(self.yAutoClicked)
+        self.yAxisUI.autoPercentSpin.valueChanged.connect(self.yAutoSpinChanged)
+        self.yAxisUI.logModeCheck.stateChanged.connect(self.yLogModeToggled)
         self.addMenu(self.yAxisUI)
 
-        self.mouseMenu = QtWidgets.QMenu("mouse mode")
-        group = QtWidgets.QActionGroup(self)
-        # This does not work! QAction _must_ be initialized with a permanent
-        # object as the parent or else it may be collected prematurely.
-        #pan = self.leftMenu.addAction("3 button", self.set3ButtonMode)
-        #zoom = self.leftMenu.addAction("1 button", self.set1ButtonMode)
-        pan = QtWidgets.QAction("pan/zoom", self.mouseMenu)
-        rect = QtWidgets.QAction("zoom box", self.mouseMenu)
-        self.mouseMenu.addAction(pan)
-        self.mouseMenu.addAction(rect)
-        pan.triggered.connect(self.setMouseModePan)
-        rect.triggered.connect(self.setMouseModeRect)
-        pan.setCheckable(True)
-        rect.setCheckable(True)
-        pan.setActionGroup(group)
-        rect.setActionGroup(group)
-        self.mouseModes = [pan, rect]
-        self.addMenu(self.mouseMenu)
-
-        self.YCursorSelect = QtWidgets.QAction("Y cursors", self, checkable=True)
-        self.YCursorSelect.triggered.connect(self.toggle_y_cursors)
-        self.addAction(self.YCursorSelect)
-        self.TCursorSelect = QtWidgets.QAction("T cursors", self, checkable=True)
-        self.TCursorSelect.triggered.connect(self.toggle_t_cursors)
-        self.addAction(self.TCursorSelect)
+        self.mouseModeUI = MouseModeMenuItem("mouse mode", self)
+        self.mouseModeUI.pan.triggered.connect(self.setMouseModePan)
+        self.mouseModeUI.rect.triggered.connect(self.setMouseModeRect)
+        self.addMenu(self.mouseModeUI)
 
         self.addLabel()
 
-        # channel select dialog
+        self.addSection("T cursors")
+        self.t_cursor_widget = CursorWidget('T1', 'T2')
+        self.t_cursor_action = QtWidgets.QWidgetAction(self)
+        self.t_cursor_action.setDefaultWidget(self.t_cursor_widget)
+        self.t_cursor_widget.T1.stateChanged.connect(self.update_t1_cursor)
+        self.t_cursor_widget.T2.stateChanged.connect(self.update_t2_cursor)
+        self.t_cursor_widget.reset.clicked.connect(self.reset_t_cursors)
+        self.addAction(self.t_cursor_action)
+
+        self.addSection("Y cursors")
+        self.y_cursor_widget = CursorWidget('Y1', 'Y2')
+        self.y_cursor_action = QtWidgets.QWidgetAction(self)
+        self.y_cursor_action.setDefaultWidget(self.y_cursor_widget)
+        self.y_cursor_widget.Y1.stateChanged.connect(self.update_y1_cursor)
+        self.y_cursor_widget.Y2.stateChanged.connect(self.update_y2_cursor)
+        self.y_cursor_widget.reset.clicked.connect(self.reset_y_cursors)
+        self.addAction(self.y_cursor_action)
+
+        self.addLabel()
+        self.addSection("add/modify/remove channels")
+        self.addLabel()
+
         self.openChannelSelectDialogButton = self.addButton("set channel list/parameters")
         self.openChannelSelectDialogButton.clicked.connect(self.channel_select_dialog)
 
-        # add channel
         self.addChannelEntry = QtWidgets.QLineEdit()
         self.addChannelEntry.setMinimumSize(300, 24)
         self.addChannelEntry.setPlaceholderText("enter channel to add")
@@ -103,7 +154,6 @@ class NDScopePlotMenu(QtWidgets.QMenu):
 
         self.addLabel()
 
-        # remove channel
         self.removeChannelList = QtWidgets.QComboBox()
         self.removeChannelList.setMinimumSize(200, 26)
         self.removeChannelList.currentIndexChanged.connect(self.remove_channel)
@@ -113,8 +163,7 @@ class NDScopePlotMenu(QtWidgets.QMenu):
         self.addAction(rcl)
 
         self.addLabel()
-        self.addSeparator()
-        self.addLabel("add/remove plots")
+        self.addSection("add/remove plots")
         self.addLabel()
 
         self.newPlotRowButton = self.addButton("add plot to row")
@@ -131,7 +180,6 @@ class NDScopePlotMenu(QtWidgets.QMenu):
         self.setContentsMargins(10, 10, 10, 10)
 
         self.view().sigStateChanged.connect(self.viewStateChanged)
-        self.updateState()
 
     ##########
 
@@ -157,43 +205,46 @@ class NDScopePlotMenu(QtWidgets.QMenu):
             self.updateState()
 
     def updateState(self):
-        # Something about the viewbox has changed; update the menu GUI
+        # something about the viewbox has changed. update the axis
+        # menu GUI
 
         state = self.view().getState(copy=False)
-        if state['mouseMode'] == self.view().PanMode:
-            self.mouseModes[0].setChecked(True)
-        else:
-            self.mouseModes[1].setChecked(True)
 
-        i = 1
-        tr = state['targetRange'][i]
-        self.yAxisUI.minText.setText("%0.5g" % tr[0])
-        self.yAxisUI.maxText.setText("%0.5g" % tr[1])
-        if state['autoRange'][i] is not False:
-            self.yAxisUI.autoRadio.setChecked(True)
-            if state['autoRange'][i] is not True:
-                self.yAxisUI.autoPercentSpin.setValue(int(state['autoRange'][i]*100))
+        # update the yAxisUI
+        # block signals in the widget while we update the values
+        with self.yAxisUI.signal_blocker():
+            # index 1 in state is y axis
+            i = 1
+            tr = tuple(map(self.plot().y_pos_to_val, state['targetRange'][i]))
+            self.yAxisUI.minText.setText("%0.5g" % tr[0])
+            self.yAxisUI.maxText.setText("%0.5g" % tr[1])
+            if state['autoRange'][i] is not False:
+                self.yAxisUI.autoRadio.setChecked(True)
+                if state['autoRange'][i] is not True:
+                    self.yAxisUI.autoPercentSpin.setValue(int(state['autoRange'][i]*100))
+            else:
+                self.yAxisUI.manualRadio.setChecked(True)
+            self.yAxisUI.logModeCheck.setChecked(state['logMode'][i])
+
+        if state['mouseMode'] == self.view().PanMode:
+            self.mouseModeUI.pan.setChecked(True)
         else:
-            self.yAxisUI.manualRadio.setChecked(True)
-        # self.yAxisUI.invertCheck.setChecked(state.get('yInverted', False))
-        # self.yAxisUI.mouseCheck.setChecked(state['mouseEnabled'][i])
-        # self.yAxisUI.visibleOnlyCheck.setChecked(state['autoVisibleOnly'][i])
-        # self.yAxisUI.autoPanCheck.setChecked(state['autoPan'][i])
+            self.mouseModeUI.rect.setChecked(True)
+
+        self.t_cursor_widget.T1.setChecked(self.plot().t_cursors.T1.isVisible())
+        self.t_cursor_widget.T2.setChecked(self.plot().t_cursors.T2.isVisible())
+
+        self.y_cursor_widget.Y1.setChecked(self.plot().y_cursors.Y1.isVisible())
+        self.y_cursor_widget.Y2.setChecked(self.plot().y_cursors.Y2.isVisible())
 
     def popup(self, pos):
         self.updateState()
 
-        ppos = self.plot().vb.mapSceneToView(pos)
+        plot = self.plot()
+
+        ppos = plot.vb.mapSceneToView(pos)
         self.pos = (ppos.x(), ppos.y())
 
-        if hasattr(self.plot(), 'y_cursors'):
-            enabled = self.plot().y_cursors is not None
-            self.YCursorSelect.setChecked(enabled)
-
-        if hasattr(self.plot(), 't_cursors'):
-            enabled = self.plot().t_cursors is not None
-            self.TCursorSelect.setChecked(enabled)
-
         # update remove channels list
         self.update_channel_list()
 
@@ -207,17 +258,7 @@ class NDScopePlotMenu(QtWidgets.QMenu):
         else:
             self.addChannelEntry.setText('')
 
-        # FIXME: only remove plot if it's not the last
-        # if numplots > 1:
-        #     self.removePlotButton.setEnabled(True)
-
-        # cparams = {chan:c.params for chan, c in self.plot().channels.items()}
-        # self.ptree.setParameters(
-        #     parameters.create_channels_params(cparams),
-        #     showTop=False,
-        # )
-
-        self.removeChannelList.setEnabled(len(self.plot().channels) > 0)
+        self.removeChannelList.setEnabled(len(plot.channels) > 0)
 
         QtWidgets.QMenu.popup(self, pos)
 
@@ -230,24 +271,16 @@ class NDScopePlotMenu(QtWidgets.QMenu):
     def reset_t0(self):
         self.plot()._reset_t0(self.pos[0])
 
-    def toggle_t_cursors(self):
-        self.plot()._update_t_cursors(
-            self.TCursorSelect.isChecked())
-
-    def toggle_y_cursors(self):
-        if self.YCursorSelect.isChecked():
-            self.plot().enable_y_cursors()
-        else:
-            self.plot().disable_y_cursors()
-
     ##########
 
     def update_channel_list(self):
         channels = list(self.plot().channels.keys())
+        self.removeChannelList.currentIndexChanged.disconnect(self.remove_channel)
         self.removeChannelList.clear()
         ls = ['remove channel'] + channels
         self.removeChannelList.addItems(ls)
         self.removeChannelList.insertSeparator(1)
+        self.removeChannelList.currentIndexChanged.connect(self.remove_channel)
 
     def validate_add(self):
         channel = str(self.addChannelEntry.text())
@@ -315,10 +348,9 @@ class NDScopePlotMenu(QtWidgets.QMenu):
 
     def yRangeTextChanged(self):
         self.yAxisUI.manualRadio.setChecked(True)
-        try:
-            self.view().setYRange(float(self.yAxisUI.minText.text()), float(self.yAxisUI.maxText.text()), padding=0)
-        except ValueError as e:
-            logging.error(f"Y range: {e}")
+        range_1 = float(self.yAxisUI.minText.text())
+        range_2 = float(self.yAxisUI.maxText.text())
+        self.plot().set_y_range((range_1, range_2))
 
     def yAutoClicked(self):
         val = self.yAxisUI.autoPercentSpin.value() * 0.01
@@ -336,3 +368,32 @@ class NDScopePlotMenu(QtWidgets.QMenu):
 
     def yInvertToggled(self, b):
         self.view().invertY(b)
+
+    def yLogModeToggled(self, state):
+        self.plot().set_log_mode(state == QtCore.Qt.Checked)
+
+    def update_t1_cursor(self):
+        self.plot().enable_t_cursors().set_visible(
+            t1=self.t_cursor_widget.T1.isChecked(),
+        )
+
+    def update_t2_cursor(self):
+        self.plot().enable_t_cursors().set_visible(
+            t2=self.t_cursor_widget.T2.isChecked(),
+        )
+
+    def reset_t_cursors(self):
+        self.plot().t_cursors.reset()
+
+    def update_y1_cursor(self):
+        self.plot().enable_y_cursors().set_visible(
+            y1=self.y_cursor_widget.Y1.isChecked(),
+        )
+
+    def update_y2_cursor(self):
+        self.plot().enable_y_cursors().set_visible(
+            y2=self.y_cursor_widget.Y2.isChecked(),
+        )
+
+    def reset_y_cursors(self):
+        self.plot().y_cursors.reset()
